@@ -14,35 +14,43 @@ import warnings
 
 warnings.filterwarnings("ignore")
 
-def feature_id(alt, sig, err, alpha, snr_factor, wct_peak_margin, log):
+def feature_id(alt, sig, sig_err, alpha, snr_factor, wct_peak_margin, log):
     
     bases = []
     tops = []
     rl_flag = []
     
-    wct_lims = [2.*snr_factor, -2.*snr_factor]
-        
+    mask_sig = (sig != sig)
+    mask_sig_err = (sig_err != sig_err)
+    sig[mask_sig] = 0.
+    sig_err[mask_sig_err] = 0.
+            
     step = np.round(alt[1] - alt[0], decimals = 5)
 
     # Wavelet covariance transform
-    wct = wavelet(sig, step, alpha)
+    wct, wct_err = wavelet(sig, sig_err, step, alpha)    
     
     # Identify bases and tops, wct criteria are applied
-    bases_dtf, tops_dtf = get_features(alt = alt, sig = sig, err = err,
-                                       wct = wct, wct_lims = wct_lims)
+    bases_dtf, tops_dtf = get_features(alt = alt, 
+                                       sig = sig, sig_err = sig_err, 
+                                       wct = wct, wct_err = wct_err, 
+                                       snr_factor = snr_factor)
     
     if tops_dtf.shape[0] > 0:
         # Combine bases and tops dataframes in a signle dataframe sorted by ascending index
         merged = combine(bases = bases_dtf, tops = tops_dtf)
             
         # Keep only base-top pairsand the first top if it exists
-        merged = match_layers(merged, wct_lims = wct_lims, 
+        merged = match_layers(merged, snr_factor = snr_factor, 
                               wct_peak_margin = wct_peak_margin)
         
         # Split merged dataframe in bases/tops lists (lenght must be the same)
         rl_flag, bases, tops = split(merged, floor = alt[0])
+        
+    wct[mask_sig] = np.nan
+    wct_err[mask_sig_err] = np.nan
                 
-    return(rl_flag, bases, tops, wct)
+    return(rl_flag, bases, tops, wct, wct_err)
     
 def geom_prop(rl_flag, bases, tops, alt, sig, log):
 
@@ -66,7 +74,7 @@ def geom_prop(rl_flag, bases, tops, alt, sig, log):
         
         sig = sig[mask]
         alt = alt[mask]
-                
+        
         for i in range(bases.shape[0]):
             
             sig_l = sig[(alt >= bases[i]) & (alt <= tops[i])]
@@ -78,7 +86,8 @@ def geom_prop(rl_flag, bases, tops, alt, sig, log):
             com[i] = np.round(np.trapz(sig_l*alt_l, x = alt_l)/
                               np.trapz(sig_l, x = alt_l), decimals = 5)
             
-            dpth[i] = np.round(np.trapz(sig_l, x = alt_l), decimals = 9)         
+            dpth[i] = np.round(np.trapz(sig_l, x = alt_l)/
+                               (tck[i]), decimals = 3)         
             
             wgh[i] = np.round(np.trapz(sig_l, x = alt_l)/
                               (np.trapz(sig, x = alt)), decimals = 5)         
@@ -86,13 +95,13 @@ def geom_prop(rl_flag, bases, tops, alt, sig, log):
             mask_max = (sig_l == np.nanmax(sig_l))
 
             peak[i] = np.round(alt_l[mask_max][-1], decimals = 5)
-            psig[i] = np.round(sig_l[mask_max][-1], decimals = 9)
-            bsig[i] = np.round(sig_l[0], decimals = 9)
-            tsig[i] = np.round(sig_l[-1], decimals = 9)
-            msig[i] = np.round(np.min([bsig[i], tsig[i]]), decimals = 9)
+            psig[i] = np.round(sig_l[mask_max][-1], decimals = 3)
+            bsig[i] = np.round(sig_l[0], decimals = 3)
+            tsig[i] = np.round(sig_l[-1], decimals = 3)
+            msig[i] = np.round(np.min([bsig[i], tsig[i]]), decimals = 3)
             shrp[i] = np.round((psig[i] - np.max([bsig[i], tsig[i]])), 
-                               decimals = 9)
-            trnd[i] = np.round((tsig[i] - bsig[i]), decimals = 9)
+                               decimals = 3)
+            trnd[i] = np.round((tsig[i] - bsig[i]), decimals = 3)
 
         # Export to xarray Data Array, ensure there are layers left after removing the insignificant ones
         if len(bases) > 0:
@@ -123,47 +132,58 @@ def geom_prop(rl_flag, bases, tops, alt, sig, log):
 
 def mask_layers(geom, alpha, log):
     
-    tck = alpha/2.
+    l_tck = alpha/2.
+
+    u_tck = 50.*alpha/2.
     
     if len(geom) > 0:
 # Remove insignificant layers 
-        mask = (geom.loc[:, 'thickness'].values >= tck)
+        mask = (geom.loc[:, 'thickness'].values >= l_tck) &\
+            (geom.loc[:, 'thickness'].values <= u_tck)
         
         geom = geom[mask]
         
     return(geom)
 
-def get_features(alt, sig, err, wct, wct_lims):
+def get_features(alt, sig, sig_err, wct, wct_err, snr_factor):
     
     # Index of potential features
     t_index = find_peaks(-wct)[0]
     b_index = find_peaks(wct)[0]
     
     # Make the error positive to be on the safe side....
-    err = np.abs(err)
-    err[err == 0.] = 1e-8
+    sig_err = np.abs(sig_err)
+    sig_err[sig_err == 0.] = 1e-8
+
+    wct_err = np.abs(wct_err)
+    wct_err[wct_err == 0.] = 1e-8
     
     # Setting dataframes of potential features
     tops_dtf = pd.DataFrame(data = np.vstack((t_index, 
                                               sig[t_index], 
+                                              sig_err[t_index], 
                                               wct[t_index],
-                                              err[t_index],
+                                              wct_err[t_index],
                                               [1]*t_index.shape[0])).T, 
-                            columns = ['index', 'sig', 'wct', 'err', 'flag'], 
+                            columns = ['index', 'sig', 'sig_err', 'wct', 'wct_err', 'flag'], 
                             index = alt[t_index], dtype = object)
     bases_dtf = pd.DataFrame(data = np.vstack((b_index, 
                                                sig[b_index], 
+                                               sig_err[b_index], 
                                                wct[b_index], 
-                                               err[b_index], 
+                                               wct_err[b_index], 
                                                [0]*b_index.shape[0])).T, 
-                             columns = ['index', 'sig', 'wct', 'err', 'flag'], 
+                             columns = ['index', 'sig', 'sig_err', 'wct', 'wct_err', 'flag'], 
                              index = alt[b_index], dtype = object)   
 
     # Filter out features with normalized wct values below the thershold
     # Sort by ascending order  
-    mask_tops = (tops_dtf.wct.values/tops_dtf.err.values < wct_lims[-1])
+    print(snr_factor)
+    mask_tops = (tops_dtf.wct.values/tops_dtf.wct_err.values < -snr_factor) &\
+        (tops_dtf.sig.values/tops_dtf.sig_err.values > snr_factor)
     
-    mask_bases = (bases_dtf.wct.values/bases_dtf.err.values > wct_lims[0])
+    mask_bases = (bases_dtf.wct.values/bases_dtf.wct_err.values > snr_factor) &\
+        (bases_dtf.sig.values/bases_dtf.sig_err.values > snr_factor)
     
     tops_dtf = tops_dtf.iloc[mask_tops, :].sort_index(axis = 0, 
                                                       ascending=True, 
@@ -171,8 +191,8 @@ def get_features(alt, sig, err, wct, wct_lims):
     
     bases_dtf = bases_dtf.iloc[mask_bases, :].sort_index(axis = 0, 
                                                          ascending=True, 
-                                                         inplace=False) 
-    
+                                                         inplace=False)
+
     return(bases_dtf, tops_dtf)
 
 def combine(bases, tops):
@@ -189,49 +209,63 @@ def combine(bases, tops):
     
     return(merged)
 
-def match_layers(merged, wct_lims, wct_peak_margin):
+def match_layers(merged, snr_factor, wct_peak_margin):
     
     # At least one base and top exist   
     # Search for a top-base pattern that signifies the change of layer and 
     # keep only those features in merged 
-    if len(merged) > 0 and (merged.flag.values == 0).any() \
-    and (merged.flag.values == 1).any:
+    flags = merged.copy().flag.values
+    wct = merged.copy().wct.values
+    wct_err = merged.copy().wct_err.values
+    index = merged.copy().index.values
+    wctnr = wct/wct_err
+    
+    if len(merged) > 0 and (flags == 0).any() and (flags == 1).any():
         layer_ind = [] 
         stack = 0
         for i in range(0, merged.shape[0] - 1):
-            pattern = [merged.flag.iloc[i], merged.flag.iloc[i+1]]
+            pattern = [flags[i], flags[i+1]]
             if pattern[0] == 1 and pattern[1] == 0:
-                max_wct = np.nanmax(merged.wct.values[stack:i+1])
-                min_wct = np.nanmin(merged.wct.values[stack:i+1])
-                max_mask = (merged.wct.values[stack:i+1] >= wct_peak_margin*max_wct)
-                min_mask = (merged.wct.values[stack:i+1] <= wct_peak_margin*min_wct)
+                wct_sl = wct[stack:i+1]
+                wctnr_sl = wctnr[stack:i+1]
+                index_sl = index[stack:i+1]
+                max_wct = np.nanmax(wct_sl)
+                min_wct = np.nanmin(wct_sl)
+                max_mask = (wct_sl >= wct_peak_margin*max_wct) &\
+                    (wctnr_sl >= snr_factor)
+                min_mask = (wct_sl <= wct_peak_margin*min_wct) &\
+                    (wctnr_sl <= -snr_factor)            
                 if max_mask.any():
-                    layer_ind.append(min(merged.index.values[stack:i+1][max_mask]))
+                    layer_ind.append(min(index_sl[max_mask]))
                 if min_mask.any():
-                    layer_ind.append(max(merged.index.values[stack:i+1][min_mask]))
+                    layer_ind.append(max(index_sl[min_mask]))
                 stack = i + 1
-        max_wct = np.nanmax(merged.wct.values[stack:])
-        min_wct = np.nanmin(merged.wct.values[stack:])
-        max_mask = (merged.wct.values[stack:] >= wct_peak_margin*max_wct)
-        min_mask = (merged.wct.values[stack:] <= wct_peak_margin*min_wct)            
+        wct_sl = wct[stack:]
+        wctnr_sl = wctnr[stack:]
+        index_sl = index[stack:]
+        max_wct = np.nanmax(wct_sl)
+        min_wct = np.nanmin(wct_sl)
+        max_mask = (wct_sl >= wct_peak_margin*max_wct) &\
+            (wctnr_sl >= snr_factor)
+        min_mask = (wct_sl <= wct_peak_margin*min_wct) &\
+            (wctnr_sl <= -snr_factor)            
         if max_mask.any():
-            layer_ind.append(min(merged.index.values[stack:][max_mask]))
+            layer_ind.append(min(index_sl[max_mask]))
         if min_mask.any():
-            layer_ind.append(max(merged.index.values[stack:][min_mask]))
+            layer_ind.append(max(index_sl[min_mask]))
 
         merged = merged.loc[layer_ind, :].sort_index(axis = 0, 
                                                      ascending=True, 
                                                      inplace=False) 
-    
     # Only tops exist          
-    if len(merged) > 0 and (merged.flag.values == 0).any() == False\
-    and (merged.flag.values == 1).any:
+    if len(merged) > 0 and (flags == 0).any() == False\
+    and (flags == 1).any():
         layer_ind = [] 
-        min_wct = np.nanmin(merged.wct.values)
-        min_mask = ((merged.wct.values <= wct_peak_margin*min_wct) & 
-                    (merged.wct.values/merged.err.values < wct_lims[-1]))            
+        min_wct = np.nanmin(wct)
+        min_mask = (wct <= wct_peak_margin*min_wct) &\
+            (wctnr <= -snr_factor)                    
         if min_mask.any():
-            layer_ind.append(max(merged.index.values[min_mask]))
+            layer_ind.append(max(index[min_mask]))
 
         merged = merged.loc[layer_ind, :].sort_index(axis = 0, 
                                                      ascending=True, 
@@ -271,4 +305,4 @@ def split(merged, floor):
         tops = np.hstack((merged.index.values[0],[]))
         rl_flag = np.hstack((1,[]))
         
-    return(rl_flag, bases, tops)
\ No newline at end of file
+    return(rl_flag, bases, tops)
